package at.jku.risc.stout.urmc;

/*
 import com.google.gwt.event.dom.client.*;
 import com.google.gwt.core.client.EntryPoint;
 import com.google.gwt.user.client.ui.*;
 import com.google.gwt.dom.client.*;
 */

import java.io.PrintStream;

import at.jku.risc.stout.urmc.algo.DebugLevel;
import at.jku.risc.stout.urmc.algo.MatchAlgorithm;
import at.jku.risc.stout.urmc.algo.MatchProblem;
import at.jku.risc.stout.urmc.algo.MatchSystem;
import at.jku.risc.stout.urmc.algo.Substitution;
import at.jku.risc.stout.urmc.data.InputParser;
import at.jku.risc.stout.urmc.data.atom.NodeFactory;
import at.jku.risc.stout.urmc.util.ControlledException;

/**
 * HelloWorld application.
 */
public class MatchGWT implements EntryPoint {

	private static final String PRINT_HEAD = "";
	private static final String PRINT_TAIL = "DONE";

	private final PrintStream out = new PrintStream(System.out) {
		StringBuilder buffer = new StringBuilder();
		long count = 0L;

		public void print(int b) {
			buffer.appendCodePoint(b);
			if (b == '\n') {
				count++;
				flush();
			}
		}

		public void print(char c) {
			buffer.append(c);
			if (c == '\n') {
				count++;
				flush();
			}
		}

		public void print(Object obj) {
			print(String.valueOf(obj));
		}

		public void println(Object obj) {
			println(String.valueOf(obj));
		}

		public void println(String str) {
			print(str);
			println();
		}

		public void print(String str) {
			buffer.append(str);
		}

		public void println() {
			print('\n');
		}

		public void flush() {
			String lineNum = "000000" + count + ": ";
			if (buffer.length() > 0)
				display(lineNum.substring(lineNum.length() - 8,
						lineNum.length())
						+ buffer.toString());
			buffer.setLength(0);
		}

		public void close() {
			flush();
			count = 0L;
		}
	};
	private final PrintStream info = new PrintStream(System.out) {
		StringBuilder buffer = new StringBuilder();

		public void print(int b) {
			buffer.appendCodePoint(b);
			if (b == '\n')
				flush();
		}

		public void print(char c) {
			buffer.append(c);
			if (c == '\n')
				flush();
		}

		public void print(Object obj) {
			print(String.valueOf(obj));
		}

		public void println(Object obj) {
			println(String.valueOf(obj));
		}

		public void println(String str) {
			print(str);
			println();
		}

		public void print(String str) {
			buffer.append(str);
		}

		public void println() {
			print('\n');
		}

		public void flush() {
			if (buffer.length() > 0)
				display("INFO:  " + buffer.toString(), "gray");
			buffer.setLength(0);
		}

		public void close() {
			flush();
		}
	};
	private final PrintStream err = new PrintStream(System.out) {
		StringBuilder buffer = new StringBuilder();

		public void print(int b) {
			buffer.appendCodePoint(b);
			if (b == '\n')
				flush();
		}

		public void print(char c) {
			buffer.append(c);
			if (c == '\n')
				flush();
		}

		public void print(Object obj) {
			print(String.valueOf(obj));
		}

		public void println(Object obj) {
			println(String.valueOf(obj));
		}

		public void println(String str) {
			print(str);
			println();
		}

		public void print(String str) {
			buffer.append(str);
		}

		public void println() {
			print('\n');
		}

		public void flush() {
			if (buffer.length() > 0)
				display("ERROR: " + buffer.toString(), "red");
			buffer.setLength(0);
		}

		public void close() {
			flush();
		}
	};

	public void btnClicked() {
		reset();
		if (PRINT_HEAD.length() > 0)
			display(PRINT_HEAD, "gray");
		try {
			String aupsIn = getParameter("problemset");
			DebugLevel debugLevel = DebugLevel.SIMPLE;
			try {
				debugLevel = DebugLevel.valueOf(getParameter("debugmode"));
			} catch (Exception ignored) {
			}
			boolean justify = "true".equals(getParameter("justify"));

			info.println("Justify computed unifiers = " + justify);
			info.println("Output format = " + debugLevel);
			info.println();
			info.println("PROBLEM:");
			MatchProblem p = new MatchProblem();
			new InputParser(p, new NodeFactory()).parseEqSystem(aupsIn, info);
			new MatchAlgorithm(new MatchSystem(p, new Substitution()))
					.match(debugLevel, out, out, justify);
		} catch (ControlledException ex) {
			err.println(ex.getMessage() + " (" + ex.getClass().getSimpleName()
					+ ")");
		} catch (Throwable ex) {
			ex.printStackTrace(err);
		}

		info.flush();
		out.flush();
		err.flush();
		display(PRINT_TAIL, "gray");
	}
	public String getParameter(String name) {
		String ret = getTextVal(name);
		if (ret == null)
			ret = getSelectVal(name);
		if (ret == null)
			ret = getCheckedVal(name);
		return ret;
	}

	// GWT Comment Start
	private String getTextVal(String name) {
		if ("problemset".equals(name))
			return "f(c), f(f(g(a, a)), a, a) =^= c, f(g(b, b, b), b, b, b)";
		if ("debugmode".equals(name))
			return DebugLevel.SIMPLE.toString();
		return null;
	}

	private String getCheckedVal(String name) {
		return null;
	}

	private String getSelectVal(String name) {
		return null;
	}

	private static void display(Object obj, String... args) {
		System.out.print(obj);
	}

	private void reset() {
	}
	// GWT Comment END

/*
	private Document gwtDoc;
	private Element gwtContainer;
	private Button gwtButton;
	private NodeCollection<Element> gwtFormElems;

	public void onModuleLoad() {
		gwtContainer = RootPanel.get().getElement();
		gwtDoc = gwtContainer.getOwnerDocument();
		gwtContainer = gwtDoc.getElementById("gwtconsole");
		gwtButton = Button.wrap(gwtDoc.getElementById("gwtbutton"));
		gwtFormElems = gwtButton.getElement().<ButtonElement> cast().getForm()
				.getElements();
		gwtButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				btnClicked();
			}
		});
	}

	private void display(Object obj, String color) {
		Element gwtContainerParent = gwtContainer;
		gwtContainer = gwtDoc.createSpanElement();
		gwtContainer.getStyle().setColor(color);
		gwtContainerParent.appendChild(gwtContainer);
		display(obj);
		gwtContainer = gwtContainerParent;
	}

	private void display(Object obj) {
		gwtContainer.appendChild(gwtDoc.createTextNode(String.valueOf(obj)));
	}

	public String getTextVal(String name) {
		Element el = gwtFormElems.getNamedItem(name);
		if (InputElement.is(el)) {
			InputElement in = InputElement.as(el);
			if ("checkbox".equalsIgnoreCase(in.getType())
					|| "radio".equalsIgnoreCase(in.getType()))
				return null;
			return in.getValue();
		}
		if (TextAreaElement.is(el))
			return TextAreaElement.as(el).getValue();
		return null;
	}

	public String getCheckedVal(String name) {
		for (int i = 0, n = gwtFormElems.getLength(); i < n; i++) {
			Element el = gwtFormElems.getItem(i);
			if (InputElement.is(el)) {
				InputElement in = InputElement.as(el);
				if (in.isChecked() && name.equals(in.getName()))
					return in.getValue();
			}
		}
		return null;
	}

	public String getSelectVal(String name) {
		Element el = gwtFormElems.getNamedItem(name);
		if (SelectElement.is(el)) {
			SelectElement sel = SelectElement.as(el);
			if (sel.getLength() == 0)
				return null;
			int selIdx = sel.getSelectedIndex();
			if (selIdx == -1 && !sel.isMultiple())
				selIdx = 0;
			OptionElement selOpt = sel.getOptions().getItem(selIdx);
			return selOpt.getValue();
		}
		return null;
	}

	public void reset() {
		info.close();
		out.close();
		err.close();
		gwtContainer.removeAllChildren();
	}
*/
	public static void main(String[] args) {
		new MatchGWT().btnClicked();
	}
}
