/*
 * Copyright 2016 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.urmc.algo;

import at.jku.risc.stout.urmc.data.TermNode;
import at.jku.risc.stout.urmc.data.atom.HedgeVar;
import at.jku.risc.stout.urmc.data.atom.Variable;

public class MatchSystem {
	private MatchProblem problem;
	private Substitution sigma;
	private MatchSystem next;

	public MatchSystem() {
		this.problem = new MatchProblem();
		this.sigma = new Substitution();
	}

	public MatchSystem(MatchProblem problem, Substitution sigma) {
		this.problem = problem;
		this.sigma = sigma;
	}

	public void apply(Substitution theta) {
		problem.apply(theta);
		sigma.compose(theta);
	}

	public void apply(Variable fromVar, TermNode toTerm) {
		problem.apply(fromVar, toTerm);
		sigma.compose(fromVar, toTerm);
	}

	public MatchProblem getProblem() {
		return problem;
	}

	public Substitution getSigma() {
		return sigma;
	}

	public MatchSystem getNext() {
		return next;
	}

	public void setNext(MatchSystem next) {
		this.next = next;
	}

	public void eliminateSeqVariable(HedgeVar seqVar) {
		apply(seqVar, TermNode.EMPTY);
	}

	public MatchSystem copy() {
		return new MatchSystem(problem.copy(), sigma.copy());
	}

	@Override
	public String toString() {
		return "<" + problem + ", " + sigma + ">";
	}
}
